/* This code is dedicated to public home use and not commercial porpuse,               |
 * so just use the code with no hesitation.                                            |
 *                                                                                     |
 * please be aware that I will waranty nothing about using this code,                  |
 * you use this on your own risk, this code has some special coding abilities          |
 * to minimize the tolerance and read as fast as possible, put it in your mind         |
 * that fast reading and precision are against each other, this code has the ability   |
 * of calibrating device automatically with precise weights, because of living in Iran |
 * you cannot even buy me a beer! so let's enjoy the life as it is.                    |
 * you do not need to change anything but symbols for other units like lb or Ounce,    |
 * just change the symbol as you desire from g to whatever unit you like, then put the |
 * precise weight on the scale and push the calibration button, simple as that.        |
 
 Features:
 1- Auto-calibration system
 2- Temperature reading sensor
 3- low noise and accurte reading
 4- tolerance compensation system
 5- EEPROM saving ability to store calibration and temperature for next use
 6- Auto zero function to keep zero steady and not drift over time and temperature change
 7- Range of measuring is from 0.3g up to 5000g
 8- use non-flickering lcd update function
 9- with Calibration temperature recording and printing ability to show the temperature 
 of the calibration moment, so, someone knows when to calibrate it again.
 */

#include <HX711.h>              // library for HX711 Loadcell ADC
#include <Wire.h> 
#include <LiquidCrystal_I2C.h>
#include <EEPROM.h>             // library for EEPROM Control Center

LiquidCrystal_I2C lcd(0x27, 16, 2);

//temperature variables
float Vin=5.0;     // [V]        
float Rt=10000;    // Resistor t [ohm]
float R0=10000;    // value of rct in T0 [ohm]
float T0=298.15;   // use T0 in Kelvin [K]
float Vout=0.0;    // Vout in A0 
float Rout=0.0;    // Rout in A0
//use the NTC datasheet to get this values.
float T1=273.15;      // [K] in datasheet 0º C
float T2=373.15;      // [K] in datasheet 100° C
float RT1=35563;      // [ohms]  resistence in T1
float RT2=549;        // [ohms]   resistence in T2
float beta=0.0;       // initial parameters [K]
float Rinf=0.0;       // initial parameters [ohm]   
float TempK=0.0;      // variable output
float TempC=0.0;      // variable output

//Input Output pins definition
const int TempInput = 1;
const int LOADCELL_DOUT_PIN = 3;
const int LOADCELL_SCK_PIN = 2;
const int TARE_Button = 11;
const int CALIBRATION_Button = 9;

HX711 scale;

//Scale variables
double p=0.00;
double y=451.06; //the scale factor 451.06 for gram in 20°C,
                 //after Calibration the number will be stored
                 //automatically in EEPROM and will be called at 
                 //each boot until the next calibration change it
                 //so the initial number of 451.06 will be used just
                 //once for each device and is no longer usable.
int Tare =0;
int Calib=0;
double Ref = 300.00; //Reference weight to calibrate Scale, here
                     //is 300 g weight, change it according to your
                     //reference
                     
const int RT=50;    //number of times to read the sensor(keep it less than 100)
const int ER=2;     //number of each individual read
                    //total number of reads = RT*ER
const int Dec=1;    //round to Dec decimals
unsigned long long CurrentMillis = 0;
unsigned long long FirstMillis = 0;
int screen = 0;              // initial screen state
int screenMax = 1;           // screen maximum states
bool screenState = true;     // initially we have a new screen, by definition

void setup() {

  //initialization of the Serial and scale
  Serial.begin(38400);
  scale.begin(LOADCELL_DOUT_PIN, LOADCELL_SCK_PIN);
  
  //Welcome messages
  lcd.begin();
  lcd.backlight();
  lcd.clear();
  lcd.setCursor(2,0);
  lcd.print(F("FEN Co. Ltd."));
  lcd.setCursor(0,1);
  lcd.print(F("::::::::::::::::"));
  delay(2000);
  
  scale.set_gain(80);
  scale.set_scale();
  scale.set_offset();
  if (!Serial.available())
  {
  Serial.print(F("Please Waite."));
  }
  lcd.setCursor(0,1);
   for(int i=0; i<16; i++){
    if (!Serial.available())
  {
    Serial.print(F("."));
  }
    lcd.setCursor(i,1);
    lcd.print(F(":"));
    if (!Serial.available())
  {
    Serial.print(F("+"));
  }
    delay(100);
  }
  if (!Serial.available())
  {
  Serial.println(F("."));
  Serial.println(F("LET's BEGIN................."));
  }
  
  lcd.clear();
  lcd.setCursor(3,0);
  lcd.print(F("Hacked by:"));
  lcd.setCursor(2,1);
  lcd.print(F("FARZAD"));
  lcd.write(0xdf);
  lcd.print(F("6532"));
  delay(2000);

  lcd.clear();
  lcd.setCursor(0,0);
  lcd.print(F("Prime Thanks to"));
  lcd.setCursor(6,1);
  lcd.print(F("YOU"));
  y = eepromread(0,y);
  delay(2000);
  
  // call the Tare Function
  Taring();
  
  //pinMode definitions
  pinMode(TARE_Button, INPUT);
  pinMode(CALIBRATION_Button, INPUT);
  pinMode(TempInput, INPUT);

  //Prime calculation of the temperature value
  beta=(log(RT1/RT2))/((1/T1)-(1/T2));
  Rinf=R0*exp(-beta/T0);

  lcd.clear();
}

void loop() {
  
  //call Scale Function
  ScaleCalc(RT,Dec);

  //call the temperature function
  Temp();

  //call button read function
  ReadButton();

  //set auto zero
  if(p<0.3 && p>(-0.3))
  AutoZero();

  //lcd screen update function
  UpdateScreen();

  /*//call Serial Print Function
  if (!Serial.available())
  {
  SerialPrinting();
  }
 
  //call serial read function
  if (Serial.available())
  {
  SerialReading();
  }*/
}
//******************************************************************************************
// Calibrating Function
void Calibration(){ //Calibrating function for 300.00 g

  double b=0.00;

  lcd.clear();
  lcd.setCursor(0,0);
  lcd.print(F("Please put the "));
  lcd.setCursor(0,1);
  lcd.print(Ref);
  lcd.print(F("g weight"));
  if (!Serial.available())
  {
  Serial.print(F("Please put the"));
  Serial.print(Ref);
  Serial.print(F(" g weight"));
  }
  
  delay(3000);
  lcd.clear();
  for(int i=0; i<15; i++){
    if (!Serial.available())
  {
    Serial.print(F("."));
  }
    lcd.setCursor(i,0);
    lcd.print(F(":"));
    int j=15-i;
    lcd.setCursor(j,1);
    lcd.print(F(":"));
    delay(200);
    
  }
  if (!Serial.available())
  {
  Serial.println(F("."));
  }
  
   Start:
   
    if (!Serial.available())
  {
    Serial.print(F("The factor is: "));Serial.println(y);
    Serial.println(F("Let's Calibrate it...."));
  }
    lcd.clear();
    lcd.setCursor(2,0);
    lcd.print(F("Factor is:"));
    lcd.setCursor(4,1);
    lcd.print(y);
    lcd.clear();
    lcd.setCursor(2,0);
    lcd.print(F("Weight is:"));
    lcd.setCursor(4,1);
    lcd.print(p,2);
    
        b=p-Ref;
        y=y+b;
        //call Scale
        ScaleCalc(RT,Dec);

        if (!Serial.available())
  {
        Serial.print(F("The factor is: "));Serial.print(y);
        Serial.print(F("          ||         "));
        Serial.print(F("The Unit is: "));Serial.println(p,2);
  }
        
        if(p<(Ref+0.05) && p>(Ref-0.05))
          goto End;

    goto Start;
   
     End:
     if (!Serial.available())
     {
      Serial.println(F("The Scale is Calibrated"));
     }
      lcd.clear();
      lcd.setCursor(1,0);
      lcd.print(F("The Scale is"));
      lcd.setCursor(2,1);
      lcd.print(F("C"));delay(200);lcd.print(F("A"));delay(200);lcd.print(F("L"));
      delay(200);lcd.print(F("I"));delay(200);lcd.print(F("B"));delay(200);
      lcd.print(F("R"));delay(200);lcd.print(F("A"));delay(200);lcd.print(F("T"));
      delay(200);lcd.print(F("E"));delay(200);lcd.print(F("D"));
      eepromwrite(0,y);
      eepromwrite(50,TempC);
      delay(2000);
      lcd.clear();
}
//******************************************************************************************
// Taring Function
void Taring(){ //Taring Function
  if (!Serial.available())
  {
  Serial.print(F("Taring"));
  }
  lcd.clear();
  lcd.setCursor(0,0);
  lcd.print(F(".....Taring....."));
      for(int i=0; i<16; i++){
        if (!Serial.available())
  {
        Serial.print(F("."));
  }
        lcd.setCursor(i,1);
        lcd.print(F(":"));
        AutoZero();
        delay(50);
      }
      if (!Serial.available())
  {
      Serial.println(F("."));
  }
  scale.tare();
  lcd.clear();
}
//******************************************************************************************
// Rounding Function
double truncate(double val, int dec) //rounding the scale value to dec precision
{
    float x = val * pow(10, dec);
    float y = round(x);
    x = y/(pow(10, dec));
    return x;
}
//******************************************************************************************
// EEPROM WRITE Function
void eepromwrite (int adrs,float Value)
{
  EEPROM.put(adrs, Value);
  
}
//******************************************************************************************
// EEPROM READ Function
float eepromread (int adrs,float Value)
{
  Value = EEPROM.get(adrs, Value);
  return Value;
}
//******************************************************************************************
// Temperature Function
void Temp(void)
{
  Vout=Vin*((float)(analogRead(1))/1024.0); // calc for ntc
  Rout=(Rt*Vout/(Vin-Vout));

  TempK=(beta/log(Rout/Rinf)); // calc for temperature
  TempC=TempK-273.15;
}
//******************************************************************************************
// Serial Read Function
void SerialReading (void)
{
  if (Serial.available())
  {
    char temp = Serial.read();
    if (temp == '+')
      y += 10.00;
    else if (temp == '-')
      y -= 10.00;
    else if (temp == '1')
      y += 1.00;
    else if (temp == '2')
      y -= 1.00;
    else if (temp == '3')
      y += 0.10;
    else if (temp == '4')
      y -= 0.10;
    else if(temp == 't'){
      Taring();
    }
    else if(temp == 'c')
    Calibration();
  }
}
//******************************************************************************************
// Serial Print Function
void SerialPrinting(void)
{
  if (!Serial.available())
  {
  //Serial.print(F("Value is: "));Serial.print(scale.get_value(30));Serial.print(F("   |   "));
  Serial.print(F("factor is: ")); Serial.print(y);Serial.print(F("   |   "));
  Serial.print(F("Weight is: ")); Serial.print(p,1);Serial.println(F("g   |   "));
  //Serial.print(F("Average is: "));Serial.println(scale.read_average(30));
  
  }
}
//******************************************************************************************
// Read Button Function
void ReadButton(void)
{
  Tare = digitalRead(TARE_Button);
  Calib = digitalRead(CALIBRATION_Button);
  if(Tare == LOW){
    if (!Serial.available())
  {
    Serial.println(F("Tare on"));
  }
    Taring();
  }
  if(Calib == LOW){
    if (!Serial.available())
  {
    Serial.println(F("Calib on"));
  }
    Calibration();
  }
}
//******************************************************************************************
// Scale Calculation Function, t stands for times of collecting data (10<t<100), 10 fast, 100 accurate
// d stands for decimals precision of the output value
void ScaleCalc(int t,int d)
{
  scale.set_scale(y);
  p=scale.get_units(t);
  p = truncate(p,d);
}
//******************************************************************************************
// Auto Zero function to keep the 0.0 steady, it does tare b times
void AutoZero()
{
    double x = scale.get_offset();
    scale.set_offset(x);
    p=0.0;
}

//******************************************************************************************
//Update LCD Screen Function
void UpdateScreen(){
  
  char float_str0[7];
  char float_str1[6];
  char float_str2[6];
  char line0[17];
  char line1[17];
  char line2[17];
  float T;

  //read the last calibration temperature recorded
  T = eepromread(50,TempC);
  
  //dtostrf to use float and double numbers in sprintf, (float, minimum width, number of decimals, buffer)
  dtostrf(p,1,1,float_str0);
  dtostrf(TempC,1,1,float_str1);
  dtostrf(T,1,1,float_str2);
  
  //sprintf to update the array for screen
  sprintf(line0,"%-16s",float_str0);
  sprintf(line1,"%-16s",float_str1);
  sprintf(line2,"%-16s",float_str2);

  // the unit in my program is "g", change it according to your unit, like "lb"
  line0[15]={'g'};
  
  // printing °C
  line1[5]={(char)223};
  line1[6]={'C'};
  line2[5]={(char)223};
  line2[6]={'C'};
  
  line2[8]={'@'};
  line2[10]={'C'};
  line2[11]={'A'};
  line2[12]={'L'};
  line2[13]={'.'};

  //updating screen
  lcd.setCursor(0,0);
  lcd.print(line0);
  lcd.setCursor(0,1);

  CurrentMillis = millis();
  
  // save the last time you changed the display 
  if(CurrentMillis - FirstMillis > 8000)
  {
    FirstMillis = CurrentMillis; 
    screen++;
    // all screens done? => start over
    if (screen > screenMax) screen = 0;                             
    screenState = true;
  }

  if (screenState)
  {
    // reset for next iteration
    screenState = false;                                         
    switch(screen)
    {
    case 0:
      lcd.print(line1);
      break;
    case 1:
      lcd.print(line2);
      break;
    }
  }
  
  Serial.println(p,Dec);
}
